<?php

namespace Newage\Advertisements\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Newage\Advertisements\Http\Requests\AdvertisementRequest;
use Newage\Advertisements\Interfaces\AdvertisementRepositoryInterface;
use Newage\Advertisements\Models\Advertisement;

/**
 * Resource controller class for advertisement.
 */
class AdvertisementResourceController extends BaseController
{

    /**
     * Initialize advertisement resource controller.
     *
     * @param type AdvertisementRepositoryInterface $advertisement
     *
     * @return null
     */
    public function __construct(AdvertisementRepositoryInterface $advertisement)
    {
        parent::__construct();
        $this->repository = $advertisement;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Newage\Advertisements\Repositories\Criteria\AdvertisementResourceCriteria::class);
    }

    /**
     * Display a list of advertisement.
     *
     * @return Response
     */
    public function index(AdvertisementRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Newage\Advertisements\Repositories\Presenter\AdvertisementPresenter::class)
                ->$function();
        }

        $advertisements = $this->repository->paginate();

        return $this->response->title(trans('advertisements::advertisement.names'))
            ->view('advertisements::advertisement.index', true)
            ->data(compact('advertisements'))
            ->output();
    }

    /**
     * Display advertisement.
     *
     * @param Request $request
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function show(AdvertisementRequest $request, Advertisement $advertisement)
    {

        if ($advertisement->exists) {
            $view = 'advertisements::advertisement.show';
        } else {
            $view = 'advertisements::advertisement.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('advertisements::advertisement.name'))
            ->data(compact('advertisement'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new advertisement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(AdvertisementRequest $request)
    {

        $advertisement = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('advertisements::advertisement.name')) 
            ->view('advertisements::advertisement.create', true) 
            ->data(compact('advertisement'))
            ->output();
    }

    /**
     * Create new advertisement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(AdvertisementRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $advertisement                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('advertisements::advertisement.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advertisements/advertisement/' . $advertisement->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/advertisements/advertisement'))
                ->redirect();
        }

    }

    /**
     * Show advertisement for editing.
     *
     * @param Request $request
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function edit(AdvertisementRequest $request, Advertisement $advertisement)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('advertisements::advertisement.name'))
            ->view('advertisements::advertisement.edit', true)
            ->data(compact('advertisement'))
            ->output();
    }

    /**
     * Update the advertisement.
     *
     * @param Request $request
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function update(AdvertisementRequest $request, Advertisement $advertisement)
    {
        try {
            $attributes = $request->all();

            $advertisement->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('advertisements::advertisement.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('advertisements/advertisement/' . $advertisement->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advertisements/advertisement/' . $advertisement->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the advertisement.
     *
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function destroy(AdvertisementRequest $request, Advertisement $advertisement)
    {
        try {

            $advertisement->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advertisements::advertisement.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('advertisements/advertisement/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('advertisements/advertisement/' . $advertisement->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple advertisement.
     *
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function delete(AdvertisementRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('advertisements::advertisement.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('advertisements/advertisement'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advertisements/advertisement'))
                ->redirect();
        }

    }

    /**
     * Restore deleted advertisements.
     *
     * @param Model   $advertisement
     *
     * @return Response
     */
    public function restore(AdvertisementRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('advertisements::advertisement.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/advertisements/advertisement'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/advertisements/advertisement/'))
                ->redirect();
        }

    }

}
